package com.chinaitop.depot.intelligent.basicdata.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.intelligent.basicdata.mapper.TQtdevinfoMapper;
import com.chinaitop.depot.intelligent.basicdata.model.TQtdevinfo;
import com.chinaitop.depot.intelligent.basicdata.model.TQtdevinfoExample;
import com.chinaitop.depot.intelligent.basicdata.service.QtdevinfoService;
import com.chinaitop.depot.intelligent.basicdata.service.WareHouseBasicInfoService;
import com.chinaitop.depot.intelligent.utils.DateUtils;
import com.chinaitop.depot.intelligent.utils.JsonToObjectUtils;
import com.chinaitop.depot.intelligent.utils.UuidUtils;
import com.unissoft.model.BasicEntity;
import com.unissoft.model.SendEntity;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class QtdevinfoServiceImpl implements QtdevinfoService {
    @Resource
    private TQtdevinfoMapper tQtdevinfoMapper;
    @Resource
    private WareHouseBasicInfoService wareHouseBasicInfoService;
    @Resource
    private UuidUtils uUidUtils;
    
    
    @Override
    public List<TQtdevinfo> getList(String cfCode) {
        TQtdevinfoExample example = new TQtdevinfoExample();
        TQtdevinfoExample.Criteria criteria = example.createCriteria();
        if (StringUtils.isNotEmpty(cfCode)) {
            criteria.andVcfcodeEqualTo(cfCode);
        }
        criteria.andDelFlagEqualTo(1);//未删除
        return tQtdevinfoMapper.selectByExample(example);
    }

    @Override
    public TQtdevinfo getById(String id) {
        return tQtdevinfoMapper.selectByPrimaryKey(id);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void save(String grainJson) {
        if (StringUtils.isNotBlank(grainJson)) {
            TQtdevinfo tQtdevinfo = (TQtdevinfo) JsonToObjectUtils.jsonToObject(grainJson, TQtdevinfo.class);
            tQtdevinfo.setUpdatetime(DateUtils.getDate());
            // 推送基础数据
            JSONObject jsonObject = SendEntity.synchronizationConvert(grainJson);
            jsonObject.put("updatetime",DateUtils.dateToString(tQtdevinfo.getUpdatetime()));
            if (StringUtils.isNotBlank(tQtdevinfo.getId())) {
                tQtdevinfoMapper.updateByPrimaryKey(tQtdevinfo);
                // 推送数据并返回结果
                SendEntity.cxfFactor().saveGasDevice(SendEntity.sendAndConvert(jsonObject, "U"));
            } else {
                // 获取Max+1值
                Map<String, Object> valueMax = wareHouseBasicInfoService.getCode("t_qtdevinfo", "sitecode", "vdevcode", null);
                // 替换值
                jsonObject.put("vdevcode",valueMax.get("column2"));
                tQtdevinfo.setId(uUidUtils.getCodeId(tQtdevinfo.getOrgId(), "t_qtdevinfo"));
                tQtdevinfo.setVdevcode(valueMax.get("column2").toString());
                tQtdevinfo.setDelFlag(1); //未删除
                tQtdevinfoMapper.insert(tQtdevinfo);
                // 推送数据并返回结果
                SendEntity.cxfFactor().saveGasDevice(SendEntity.sendAndConvert(jsonObject, "I"));
            }
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int deleteById(String id, String qtDeviInfoJson) {
        TQtdevinfo tQtdevinfo = (TQtdevinfo) JsonToObjectUtils.jsonToObject(qtDeviInfoJson, TQtdevinfo.class);
        tQtdevinfo.setDelFlag(0); //删除
        int i = tQtdevinfoMapper.updateByPrimaryKeySelective(tQtdevinfo);
        if (i > 0) {
            // 推送同步数据
            JSONObject jsonObject = SendEntity.synchronizationConvert(qtDeviInfoJson);
            SendEntity.cxfFactor().saveGasDevice(SendEntity.sendAndConvert(jsonObject, "D"));
        }
        return i;
    }

    @Override
    public boolean getBycfCode(String vdevcode, String vdevname) {
        TQtdevinfoExample example = new TQtdevinfoExample();
        TQtdevinfoExample.Criteria criteria = example.createCriteria();
        if (StringUtils.isNotEmpty(vdevcode)) {
            criteria.andVdevcodeEqualTo(vdevcode);
        }
        if (StringUtils.isNotEmpty(vdevname)) {
            criteria.andVdevnameEqualTo(vdevname);
        }
        List<TQtdevinfo> tQtdevinfos = tQtdevinfoMapper.selectByExample(example);
        if (tQtdevinfos.size() > 0) {
            return true;
        }
        return false;
    }

    @Transactional
    @Override
    public String synchronizationAll(String orgId) {
        //获取所有基础数据
        String device = SendEntity.cxfFactor().getGasDevice();
        //转换对象
        BasicEntity basicEntity = (BasicEntity) JsonToObjectUtils.jsonToObject(device, BasicEntity.class);
        if (basicEntity.getState().equals("0")) {
            List<Object> data = basicEntity.getData();
            if (data.size() > 0) {
                String tt = JSONObject.toJSONString(data);
                ArrayList<Object> objects = JsonToObjectUtils.jsonToObjectsDiverse(tt, TQtdevinfo.class,"yyyy-MM-dd'T'HH:mm:ss");
                objects.forEach(t -> {
                    TQtdevinfo tc = (TQtdevinfo) t;
                    tc.setId(uUidUtils.getCodeId(orgId, "t_qtdevinfo"));
                    tc.setOrgId(orgId);
                    tc.setDelFlag(1); //未删除
                    String vdevcode = tc.getVdevcode();
                    TQtdevinfo info1 = getVinfo(vdevcode);
                    if (info1 != null) {
                        tc.setId(info1.getId());
                        tQtdevinfoMapper.updateByPrimaryKey(tc);
                    } else {
                        tQtdevinfoMapper.insert(tc);
                    }
                });
            }
        } else {
            return basicEntity.getMsg();
        }
        return basicEntity.getMsg();
    }

    public TQtdevinfo getVinfo(String vdevcode) {
        TQtdevinfoExample example = new TQtdevinfoExample();
        TQtdevinfoExample.Criteria criteria = example.createCriteria();
        if (StringUtils.isNotEmpty(vdevcode)) {
            criteria.andVdevcodeEqualTo(vdevcode);
        }
        List<TQtdevinfo> infos = tQtdevinfoMapper.selectByExample(example);
        if (infos.size() > 0) {
            return infos.get(0);
        }
        return null;
    }

}
