package com.chinaitop.depot.intelligent.devControl.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chinaitop.depot.intelligent.devControl.mapper.TDevControlMapper;
import com.chinaitop.depot.intelligent.devControl.model.TDevControl;
import com.chinaitop.depot.intelligent.devControl.model.TDevControlPageParam;
import com.chinaitop.depot.intelligent.devControl.service.TDevControlService;
import com.chinaitop.depot.intelligent.utils.UuidUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * <p>
 * 设备控制 服务实现类
 * </p>
 *
 * @author root
 * @since 2024-05-09
 */
@Service
public class TDevControlServiceImpl extends ServiceImpl<TDevControlMapper, TDevControl> implements TDevControlService {
    @Resource
    private TDevControlMapper devControlMapper;

    @Override
    public Page<TDevControl> selectPageList(TDevControlPageParam devControl) {
        QueryWrapper<TDevControl> wrapper = new QueryWrapper<>();
        wrapper.eq("org_id", devControl.getOrgId());
        if (StringUtils.isNotEmpty(devControl.getCompanyName()))
            wrapper.like("company_name", devControl.getCompanyName());
        if (StringUtils.isNotEmpty(devControl.getDevName()))
            wrapper.like("dev_name", devControl.getDevName());
        return devControlMapper.selectPage(new Page<>(devControl.getCurrent(), devControl.getSize()), wrapper);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int saveData(TDevControl devControl) {
        // 生成32id
        devControl.setId(UuidUtils.getCode());
        return devControlMapper.insert(devControl);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateData(TDevControl devControl) {
        return devControlMapper.updateById(devControl);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteData(String id) {
        return devControlMapper.deleteById(id);
    }
}
