package com.chinaitop.depot.intelligent.grainsituation.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.chinaitop.depot.intelligent.grainsituation.mapper.TTestdataLayerMapper;
import com.chinaitop.depot.intelligent.grainsituation.mapper.TTestdataMapper;
import com.chinaitop.depot.intelligent.grainsituation.mapper.TTestdataPointsMapper;
import com.chinaitop.depot.intelligent.grainsituation.model.*;
import com.chinaitop.depot.intelligent.grainsituation.service.TemperatureRecordService;
import com.chinaitop.depot.intelligent.utils.DateUtils;
import com.chinaitop.depot.intelligent.utils.JsonToObjectUtils;
import com.chinaitop.depot.intelligent.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
public class TemperatureRecordServiceImpl implements TemperatureRecordService {
    @Resource
    private TTestdataMapper tTestdataMapper;

    @Resource
    private TTestdataPointsMapper tTestdataPointsMapper;

    @Resource
    private TTestdataLayerMapper tTestdataLayerMapper;

    //这三个库需要特殊处理
    private final static String []SPECIAL_ORG = {"68", "161", "71"};

    @Override
    public List<TTestdata> getList(String vCfCode, String vDatatime, String startTime, String endTime, String tType, String orgId) {
        TTestdataExample testdataExample = new TTestdataExample();
//        testdataExample.setDistinct(true);
        TTestdataExample.Criteria criteria = testdataExample.createCriteria();
        if (StringUtils.isNotEmpty(vCfCode)) {
            criteria.andStorehouseEqualTo(vCfCode);
        }
        if (StringUtils.isNotEmpty(vDatatime)) {
            criteria.andTTimeBetween(DateUtils.stringToData(vDatatime + DateUtils.startTime), DateUtils.stringToData(vDatatime + DateUtils.endTime));
        }
        if (StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime)) {
            criteria.andTTimeBetween(DateUtils.stringToData(startTime + DateUtils.startTime), DateUtils.stringToData(endTime + DateUtils.endTime));
        }
        if (ParameterUtil.isnull(startTime) && StringUtils.isNotBlank(endTime)) {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date date = null;
            try {
                date = dateFormat.parse(endTime);
            } catch (ParseException e) {
                e.printStackTrace();
            }
            criteria.andTTimeLessThanOrEqualTo(date);
        }
        if (StringUtils.isNotEmpty(tType)) {
            criteria.andIlqYqEqualTo(tType);
        }
        if (StringUtils.isNotEmpty(orgId)) {
            criteria.andOrgIdEqualTo(orgId);
        }
        testdataExample.setOrderByClause("t_time desc");
        return tTestdataMapper.selectByExample(testdataExample);
    }


    @Override
    public List<TTestdataLayer> findByHouseAndTime(String lqId, String vCfCode, String vDatatime, String orgId) {
        TTestdataLayerExample tTestdataLayerExample = new TTestdataLayerExample();
        tTestdataLayerExample.setDistinct(true);
        TTestdataLayerExample.Criteria criteria = tTestdataLayerExample.createCriteria();
        if (StringUtils.isNotBlank(orgId))
            criteria.andOrgIdEqualTo(orgId);
        if (StringUtils.isNotBlank(lqId))
            criteria.andLqIdEqualTo(lqId);
        /*if (StringUtils.isNotBlank(vCfCode))
            criteria.andLhouseEqualTo(vCfCode);
        if (StringUtils.isNotBlank(vDatatime))
            criteria.andLtimeEqualTo(DateUtils.stringToData(vDatatime));*/
//        boolean result = ArrayUtils.contains(SPECIAL_ORG, orgId);
//        if (result) {
//        	tTestdataLayerExample.setOrderByClause("layerNumber ASC");
//        } else {
//        	tTestdataLayerExample.setOrderByClause("layerNumber DESC");
//        }
        
        tTestdataLayerExample.setOrderByClause("layerNumber ASC");
        
        List<TTestdataLayer> tTestdata = tTestdataLayerMapper.selectByExample(tTestdataLayerExample);
        return tTestdata;
    }

    @Override
    public ArrayList<Object> findByHouseAndTimes(String lqId, String vCfCode, String vDatatime) {
        TTestdata byId = tTestdataMapper.getById(lqId).get(0);
        String s = byId.gettPoints();

//        boolean result = ArrayUtils.contains(SPECIAL_ORG, byId.getOrgId());
//
//        if (result) {
//        	result = false;
//        } else {
//        	result = true;
//        }
//        ArrayList<TPoints> tPoints = restZ(s, byId.getOrgId(), result);
        
        List<TPoints> tPoints = JSON.parseArray(s, TPoints.class);

        ArrayList<Object> objects = Lists.newArrayList();
        if (tPoints.size() > 0) {
            Map<Integer, Map<Integer, Map<Integer, List<TPoints>>>> collect1 = tPoints.stream().collect(Collectors.groupingBy(t -> Integer.parseInt(t.getZ()), Collectors.groupingBy(t -> Integer.parseInt(t.getX()), Collectors.groupingBy(t -> Integer.parseInt(t.getY())))));
            for (Map.Entry<Integer, Map<Integer, Map<Integer, List<TPoints>>>> entryG : collect1.entrySet()) {
                //每层数据
                Map<Integer, Map<Integer, List<TPoints>>> layerGroupV = entryG.getValue();
                for (Map.Entry<Integer, Map<Integer, List<TPoints>>> rows : layerGroupV.entrySet()) {
                    HashMap<Object, Object> objectObjectHashMap = Maps.newLinkedHashMap();

                    //每一行数据
                    Map<Integer, List<TPoints>> value = rows.getValue();
                    int a = 1;
                    objectObjectHashMap.put(a, entryG.getKey() + "-" + rows.getKey());
                    for (Map.Entry<Integer, List<TPoints>> orws : value.entrySet()) {
                        //列数据
                        List<TPoints> column = orws.getValue();
                        TPoints t = column.get(0);
                        objectObjectHashMap.put(++a, t.getTemp());
                    }
                    objects.add(objectObjectHashMap);
                }
            }
        }
        return objects;
    }

    @Override
    public List<TTestdata> getComparison(String vCfCodeLeft, String startTimeLeft, String endTimeLeft, String vCfCodeRight, String startTimeRight, String endTimeRight, String tType, String orgId) {
        TTestdataExample example = new TTestdataExample();
        example.setDistinct(true);
        TTestdataExample.Criteria criteria = example.createCriteria();
        // 左图查询条件
        if (StringUtils.isNotBlank(vCfCodeLeft)) {
            criteria.andStorehouseEqualTo(vCfCodeLeft);
        }
        // 右图查询条件
        if (StringUtils.isNotBlank(vCfCodeRight)) {
            criteria.andStorehouseEqualTo(vCfCodeRight);
        }
        if (StringUtils.isNotBlank(orgId)) {
            criteria.andOrgIdEqualTo(orgId);
        }
        // 时间条件
        if (StringUtils.isNotBlank(startTimeLeft) && StringUtils.isNotBlank(endTimeLeft)) {
            criteria.andTTimeBetween(DateUtils.stringToData(startTimeLeft + DateUtils.startTime), DateUtils.stringToData(endTimeLeft + DateUtils.endTime));
        } else if (StringUtils.isNotBlank(startTimeRight) && StringUtils.isNotBlank(endTimeRight)) {
            criteria.andTTimeBetween(DateUtils.stringToData(startTimeRight + DateUtils.startTime), DateUtils.stringToData(endTimeRight + DateUtils.endTime));
        } else {
            criteria.andTTimeBetween(DateUtils.stringToData(DateUtils.formatDate(DateUtils.getDate()) + DateUtils.startTime), DateUtils.stringToData(DateUtils.formatDate(DateUtils.getDate()) + DateUtils.endTime));
        }
        // 检测类型0测温，1粮油,2测水分
        criteria.andIlqYqEqualTo(tType);
        example.setOrderByClause("t_time desc");
        return tTestdataMapper.selectByExample(example);
    }

    @Transactional
    @Override
    public void insertBatch(List<TTestdata> testData) {
        tTestdataMapper.insertBatch(testData);
    }

    @Override
    public List<TTestdata> getByCodeAndTimes(List<String> cDcsCodes, List<Date> cTimes, List<String> lqTypes) {
        TTestdataExample example = new TTestdataExample();
        TTestdataExample.Criteria criteria = example.createCriteria();
        if ((cDcsCodes != null && cDcsCodes.size() > 0) && (cTimes != null && cTimes.size() > 0) && (lqTypes != null && lqTypes.size() > 0)) {
            criteria.andStorehouseIn(cDcsCodes);
            criteria.andTTimeIn(cTimes);
            criteria.andIlqYqIn(lqTypes);
            List<TTestdata> tTestdata = tTestdataMapper.selectByExample(example);
            return tTestdata;
        }
        return null;
    }

    @Override
    public String getOrgId(String cfCode) {
        return tTestdataMapper.getOrgId(cfCode);
    }

    @Override
    public TTestdata getById(String id) {
        TTestdataExample example = new TTestdataExample();
        TTestdataExample.Criteria criteria = example.createCriteria();
        criteria.andIdEqualTo(id);
        TTestdata tTestdata = tTestdataMapper.selectByExample(example).get(0);
        return tTestdata;
    }

    @Override
    public Map<Integer, Map<Integer, Map<Integer, List<TPoints>>>> findByTempTable(String id) {
    	TTestdata str = tTestdataMapper.findByTempTable(id);
        if (StringUtils.isBlank(str.gettPoints()))
            return null;
        
        List<TPoints> tPoints = null;
//        boolean result = ArrayUtils.contains(SPECIAL_ORG, str.getOrgId());
//    	if (result) {
//    		//把层数(Z)颠倒过来
//    		tPoints = restZ(str.gettPoints(), str.getOrgId(), result);
//    	} else {
//    		// json字符串数组转list对象
//    		tPoints = JSONArray.parseArray(str.gettPoints(), TPoints.class);
//    	}
    	tPoints = JSONArray.parseArray(str.gettPoints(), TPoints.class);
    	Map<Integer, Map<Integer, Map<Integer, List<TPoints>>>> collect = tPoints.stream()
    			.collect(Collectors.groupingBy(t -> Integer.parseInt(t.getZ())
    					, Collectors.groupingBy(t -> Integer.parseInt(t.getX())
    							, Collectors.groupingBy(t -> Integer.parseInt(t.getY())))));
		return collect;
    }

    @Override
    public Map<Integer,List<TPoints>> shallowCircularPointTempTable(String id) {
    	TTestdata str = tTestdataMapper.findByTempTable(id);
        if (StringUtils.isBlank(str.gettPoints()))
            return null;
        List<TPoints> tPoints = JSONArray.parseArray(str.gettPoints(), TPoints.class);
        // 按z层分组
        Map<Integer, List<TPoints>> collect = tPoints.stream()
                .collect(Collectors.groupingBy(t -> Integer.parseInt(t.getZ())));
        return collect;
    }

    @Override
    public void test(String orgId, Integer pageSize) throws IOException {

        if (null != pageSize) PageHelper.startPage(1, pageSize);

        Map<String, Object> param = new HashMap<>();
        if (StringUtils.isNotBlank(orgId)) param.put("orgId", orgId);
        List<Map<String, Object>> list = tTestdataPointsMapper.test(param);

        PageInfo<Map<String, Object>> pageInfo = new PageInfo<Map<String, Object>>(list);
        List<Map<String, Object>> new_list = pageInfo.getList();

        List<TTestdataPoints> t_list = new ArrayList<>();
        StringBuffer sbf = new StringBuffer(2000);

        new_list.forEach(obj -> {
            Map<String, Object> map = obj;
            String id = map.get("id").toString();
            String obj_orgId = map.get("org_id").toString();
            String points = map.get("t_points").toString();
            points = points.trim().replace("},{", "}|{")
                    .replace("[", "")
                    .replace("]", "")
                    .replace("{", "")
                    .replace("}", "")
                    .replace("\"", "")
                    .replace("temp:", "")
                    .replace("x:", "")
                    .replace("y:", "")
                    .replace("z:", "");
            TTestdataPoints tpoints = new TTestdataPoints();
            tpoints.setId(id);
            tpoints.setOrgId(obj_orgId);
            tpoints.setCfbh(map.get("storehouse").toString());
            Date time = ParameterUtil.string2datetime(map.get("t_time").toString());
            tpoints.setPointsTime(time);
            tpoints.setStrPoints(points);
            tTestdataPointsMapper.insertSelective(tpoints);
        });

    }

    /**
     * Z的坐标是否需要颠倒
     * @param t_Points 原点位值
     * @param orgId 组织机构ID
     * @param flag 是否颠倒
     * @return
     */
    private ArrayList<TPoints> restZ(String t_Points, String orgId, boolean flag) {
    	
    	// 层倒叙 最小变最大, 最大变最小
        List<TPoints> tP = JSON.parseArray(t_Points, TPoints.class);
        // 数据总长度
        int size = tP.size();//测温点总数
        // 层分组 总层数
        Map<Integer, List<TPoints>> collect = tP.stream().collect(Collectors.groupingBy(point -> Integer.parseInt(point.getZ())));
        Integer sizeZ = collect.size();//总层数
        int i = 1; // 计数初始变量
        int y = size / sizeZ; // 每层测温点的数量
        ArrayList<TPoints> tPoints = Lists.newArrayList();
        for (Integer integer : collect.keySet()) {
            List<TPoints> tPoints1 = collect.get(integer);//获取当前层的所有点位
            for (TPoints tPoint : tPoints1) {//遍历这层的这些点位
            	if (flag) {
            		tPoint.setZ(sizeZ.toString()); // 倒叙放入层值
            		if (i == y) { // 每层有多少个点
            			sizeZ--;
            		}
            		i++;
            		if (i > y) { // 到达每层点将初始计数初始化,执行替换为下一个sizeZ--
            			i = 1;
            		}
            	}
                tPoints.add(tPoint); // 重新添加到list
            }
        }

    	return tPoints;
    }
}
